-- Tank Game (Working Title)
-- By: Ensayia (ensayia@gmail.com)
-- Created on the LOVE framework (http://love2d.org)

-- main.lua

require 'lib/TEsound'	-- Sound library.
require 'lib/TSerial'		-- Serialization Libarary.

require 'help'				-- A set of helpful functions.
require 'resource'		-- Handles reference to all external game resources.

require 'block'			-- Handles creation and drawing of collidable platforms.
require 'camera'		-- Handles camera player tracking and updating.
require 'editor'			-- Handles level editor.
require 'enemy'			-- Handles enemy creation, update, and drawing.
require 'level'			-- Handles level loading.
require 'player'			-- Handles player and some things player reacts with.

-- If something isn't explained well here in the main.lua file, it is probably
-- explained better in it's own file. These functions are broken up over
-- several files for convenience.

function love.load()
	-- Set screen coords so we can reference them. At the moment these are
	-- manual and do not actually get screen size from the OS.
	screen = {x = 800, y = 600}
	pause = false
	
	-- Load a level.
	level.load()
	
	-- Turn off the system cursor, custom cursor is drawn in player:update().
	love.mouse.setVisible(false)
end

function love.update(dt)
	fps = love.timer.getFPS()

	if not pause then player.update(dt) end
	if not pause then enemy.update(dt) end
	if not pause then camera.update(dt) end
	if edit.on then edit.update(dt) end

	TEsound.cleanup()
end

function love.draw()
	-- Everything within push and pop is affected by relative camera position. Always
	-- make sure to compensate for the camera if necessary when performing
	-- calculations on entities, if not it can lead to visual weirdness.
	love.graphics.push()
	
		-- Move evertyhing (visually) relative to the camera's position. The camera tracks
		-- the player by defualt.
		love.graphics.translate(camera.x, camera.y)
		
		-- Call the draw loops of all supporting classes.
		enemy.draw()
		block.draw()	
		player.draw()
		if edit.on then edit.draw() end
	
	love.graphics.pop()
	
	-- Anything beyond pop will not get adjusted for the camera. This is good for
	-- things like a HUD.
	player.draw2()		
		
	if edit.on then edit.draw2() end
	
end

function love.mousepressed(x, y, button)	
	if not pause then player.mouse(x, y, button) end
	
	if edit.on then edit.mouse(x, y, button) end
end

function love.keypressed(key)
	if key == 'e' then edit.on = not edit.on pause = not pause edit.setBrush() end
	
	if key == 'r' then love.filesystem.load("main.lua")() love.load() end
	
	if edit.on then edit.keyboard(key) end
	
	if not pause then player.keyboard(key) end
end