-- Tank Game (Working Title)
-- By: Ensayia (ensayia@gmail.com)
-- Created on the LOVE framework (http://love2d.org)

-- player.lua


-- Define a table to contain player.
player = {}

-- Table to hold all player projectiles.
player.bullets = {}

-- Initialize player variables. This section will likely get very large.
player.health = 10
player.x = 150
player.y = 300
player.vx = 0
player.vy = 0
player.gravity = 500
player.friction = false
player.canMove = true
player.wheelRot = 0
player.gunRot = math.pi/2
player.gunMode = 1
player.grappleStage = 0
player.g1x = 0
player.g1y = 0
player.g1Rot = 0
player.g2x = 0
player.g2y = 0
player.g2Rot = 0
player.gMidX = 0
player.gMidY = 0
player.gax = 0
player.gay = 0
player.gflash = 0
player.cursorRot = 0
player.inAir = false
player.grappleSpot = true
player.collideX = false
player.collideY = false
player.imgBody = image.player.body
player.imgGun = image.player.gun
player.imgWheelA = image.player.wheelA
player.imgWheelB = image.player.wheelB
player.flashing = false
player.flashcount = 0
player.soundTimeout1 = true


-- Draw loop handles player, bullet, and cursor drawing.
function player.draw()
	if player.flashing == true then love.graphics.setColor(HSL(0, 127, player.flashcount)) else love.graphics.setColor(255, 255, 255, 255) end
	-- Draw player stuff.
	love.graphics.draw(player.imgBody, player.x, player.y , 0, 1, 1, 32, 16)
	love.graphics.draw(player.imgGun, player.x, player.y-10, player.gunRot, 1, 1, 16, 26)
	love.graphics.draw(player.imgWheelA, player.x - 19, player.y + 10, player.wheelRot, 1, 1, 8, 8)
	love.graphics.draw(player.imgWheelA, player.x + 19, player.y + 10, player.wheelRot, 1, 1, 8, 8)
	love.graphics.draw(player.imgWheelB, player.x, player.y + 13, player.wheelRot, 1, 1, 8, 8)
	
	love.graphics.setColor(255, 255, 255, 255)
	
	if player.grappleStage == 1 then 
		love.graphics.draw(image.player.hook, player.g1x, player.g1y, player.g1Rot, 1, 1, 8, 8)
		love.graphics.line(player.x, player.y, player.g1x, player.g1y)
	end
	
	if player.grappleStage == 2 then
		love.graphics.draw(image.player.hook, player.g1x, player.g1y, player.g1Rot, 1, 1, 8, 8)
		love.graphics.draw(image.player.hook, player.g2x, player.g2y, player.g2Rot, 1, 1, 8, 8)
		love.graphics.line(player.x, player.y, player.g1x, player.g1y)
		love.graphics.line(player.x, player.y, player.g2x, player.g2y)
		love.graphics.setColor(0, 96, 96, player.gflash)
		love.graphics.circle('line', player.gMidX, player.gMidY, 8, 16)
		love.graphics.setColor(255,255,255,255)
	end
	
	-- Draw bullets.
	for k,v in ipairs(player.bullets) do
			love.graphics.draw(image.player.bullet, v.x, v.y, 0, 1, 1, 4, 4)
	end
	
	-- Draw cursor.
	love.graphics.draw(image.player.cursor, love.mouse.getX() - camera.x, love.mouse.getY() - camera.y, player.cursorRot, 1, 1, 8, 8)
end

function player.draw2()
	-- HUD stuff here.
end

-- Player shooting. Called by mouse click.
function player.shoot(mx, my)
	local x,y = player.x, player.y
	
	-- Important code here. Stops the player from shooting outside of the gun rotation range.
	if my > player.y + camera.y then my = player.y + camera.y end
	
	-- Calculate a normal and store it with the bullet for direction.
	local ax, ay = math.normalize((mx - camera.x) - x, (my - camera.y) - y)
	
	-- The extra math in x and y make the bullet spawn from the tip of the barrel
	-- instead of inside the tank. A big thank you to SoggyWaffles!
	if player.gunMode == 1 then
		table.insert(player.bullets, {x = x + (ax * 30), y = y-10 + (ay * 30), nx = ax, ny = ay})
	elseif player.gunMode == 2 then
		table.insert(player.bullets, {x = x + (ax * 30), y = y-10 + (ay * 30), nx = ax-.1, ny = ay-.15})
		table.insert(player.bullets, {x = x + (ax * 30), y = y-10 + (ay * 30), nx = ax, ny = ay})
		table.insert(player.bullets, {x = x + (ax * 30), y = y-10 + (ay * 30), nx = ax-.1, ny = ay+.15})
	end
	TEsound.play(sound.player.shot)
end

-- Player update loop. Using dt.
function player.update(dt)
	if dt > 0.016 then dt = 0.016 end

	--Initialize several variables at the beginning of each update.
	player.collideY = false
	player.collideX = false
	player.grappleSpot = false

	-- Define movement keys.
	local lt = love.keyboard.isDown('a') or love.keyboard.isDown('left')
	local rt = love.keyboard.isDown('d') or love.keyboard.isDown('right')
	
	-- Update cursor rotation.
	player.cursorRot = player.cursorRot + (3 * dt)
	
	-- Do not allow the gun to rotate beyond a certain distance.
	player.gunRot = math.clamp(-1.57, -(math.atan2(player.x - (love.mouse.getX() - camera.x), player.y - (love.mouse.getY() - camera.y) )), 1.57)
	
	if math.abs(player.vx) < .1 then player.vx = 0 end
	
	if math.abs(player.vy) > 0 then
		player.inAir = true
		player.friction = false
		player.soundTimeout1 = true
	else 
		player.inAir = false
	end
	
	-- Increase or decrease player x velocity if not grappling and not in the air.
	if player.inAir == false and player.canMove == true then
		if lt then
			player.friction = false
			if player.vx > 0 then player.vx = player.vx - (450 * dt) else player.vx = player.vx - (150 * dt) end
			if player.vx < -200 then player.vx = -200 end
			player.wheelRot = player.wheelRot - 6 * dt
		elseif rt then
			player.friction = false
			if player.vx < 0 then player.vx = player.vx + (450 * dt) else player.vx = player.vx + (150 * dt) end
			if player.vx > 200 then player.vx = 200 end
			player.wheelRot = player.wheelRot + 6 * dt
		else
			player.friction = true
		end		
	end
	
	-- Apply gravity to velocities.
	player.vy = player.vy + (player.gravity * dt)
	
	-- Apply friction to velocities.
	if player.friction == true then
		if player.vx > 0 then
			player.vx = player.vx - (275 * dt)
		elseif player.vx < 0 then 
			player.vx = player.vx + (275 * dt)
		end
	end
	
	-- This collision code checks whether the player is in bounds of a platform and
	-- reacts according to where the player touched the platform.
	for k,v in ipairs(platform) do
		if player.x + 32 + (player.vx * dt) > v.x and player.x - 32 + (player.vx * dt) < v.x + v.w and player.y + 16 + (player.vy * dt) > v.y and player.y - 16 + (player.vy * dt) < v.y + v.h then
			if player.x + 32 > v.x and player.x - 32 < v.x + v.w then
				if player.y + 16 > v.y then
					player.collideY = true
					player.vy = 0
					if player.soundTimeout1 == true then
						TEsound.play(sound.player.collide)
						player.soundTimeout1 = false
					end
				elseif player.y + 16 < v.x + v.h then
					player.vy = 0
					if player.soundTimeout1 == true then
						TEsound.play(sound.player.collide)
						player.soundTimeout1 = false
					end
				end
			elseif player.y + 16 > v.y and player.y - 16 < v.y + v.h then
				if player.x + 32 > v.x then
					player.collideX = true
					player.friction = true
					player.vx = 0
					if player.soundTimeout1 == true then
						TEsound.play(sound.player.collide)
						player.soundTimeout1 = false
					end
				elseif player.x - 32 < v.x + v.h then
					player.collideX = true
					player.friction = true
					player.vx = 0
					if player.soundTimeout1 == true then
						TEsound.play(sound.player.collide)
						player.soundTimeout1 = false
					end
				end
			end
		end
		
		-- Check for bullet collision with platforms. Remove bullets that collide.
		for l,w in ipairs(player.bullets) do
			if w.x > v.x and w.x < v.x + v.w and w.y > v.y and w.y < v.y + v.h then
				table.remove(player.bullets, l)
			end
		end
	end
	
	-- Update player x and y coordinates based on velocity calculations.
	if player.collideX == false then player.x = player.x + (player.vx * dt) end
	if player.collideY == false then player.y = player.y + (player.vy * dt) end
	
	-- Set a few absolute limits on velocity
	if player.vx > 450 then player.vx = 450 end
	if player.vx < -450 then player.vx = -450 end
	if player.vy > 450 then player.vy = 450 end
	if player.vy < -450 then player.vy = -450 end
	
	-- Update bullet movement.
	for k,v in ipairs(player.bullets) do
		if v.x > screen.x - camera.x or v.x < screen.x - screen.x - camera.x then table.remove(player.bullets, k) end
		if v.y > screen.y - camera.y or v.y < screen.y - screen.y - camera.y then table.remove(player.bullets, k) end
		v.x = v.x + v.nx * 600 * dt
		v.y = v.y + v.ny * 600 * dt
	end
	
	-- Check if the mouse cursor is over a platform. If so, allow the player to grapple.	
	for k,v in ipairs(platform) do
		if love.mouse.getX() - camera.x > v.x and love.mouse.getX() - camera.x < v.x + v.w and love.mouse.getY() - camera.y > v.y and love.mouse.getY() - camera.y < v.y + v.h then
			if math.abs(player.vx) < 5 then
				player.grappleSpot = true
			end
		end
	end

	if player.flashing == true then player.flashcount = player.flashcount + (1500 * dt) if player.flashcount > 255 then player.flashcount = 0 player.flashing = false end end
	player.gflash = player.gflash + (1500 * dt)
	if player.gflash > 255 then player.gflash = 0 end
end

-- Player grappling handler. Grappling handles in three stages, the first two setting the hooks and the third
-- to launch the player.
function player.grapple(mx,my)

	-- First Stage
	if player.grappleStage == 0 then
	
		-- If the mouse is over a grapple spot and we are not in the air.
		if player.grappleSpot == true and player.inAir == false then
		
			-- Disallow movement keys and cancel player x velocity.
			player.canMove = false
			player.vx = 0
		
			-- Store the mouse's x and y to a coordinate for later calculations.
			player.g1x = mx - camera.x
			player.g1y = my - camera.y
			
			-- Store the grapple's rotation for visual consisitancy,
			player.g1Rot = -math.atan2(player.x - (love.mouse.getX() - camera.x), player.y - (love.mouse.getY() - camera.y))
			
			-- Advance to the next stage
			player.grappleStage = 1
			TEsound.play(sound.player.grapple)
		end
		
	-- Second Stage. A few things that triggered in Stage 1 do not need to be triggered again.
	elseif player.grappleStage == 1 then
		
		-- If the mouse is over a grapple spot and we are not in the air.
		if player.grappleSpot == true and player.inAir == false then
			
			-- Store the mouse's x and y to a coordinate for later calculations.
			player.g2x = mx - camera.x
			player.g2y = my - camera.y
			
			-- Store the grapple's rotation for visual consisitancy,
			player.g2Rot = -math.atan2(player.x - (love.mouse.getX() - camera.x), player.y - (love.mouse.getY() - camera.y))
			
			-- Advance to the next stage
			player.grappleStage = 2
			TEsound.play(sound.player.grapple)
		end
		
		-- Calculate the midpoint between the two grappled points.
		player.gMidX, player.gMidY = math.midpoint(player.g1x, player.g2x, player.g1y, player.g2y)
		
		-- Calculate the normal between the midpoint and the player.
		player.gax, player.gay = math.normalize(player.gMidX - player.x, player.gMidY - player.y)
		
	-- Third Stage
	elseif player.grappleStage == 2 then
		
		-- Turn off friction
		player.friction = false
		
		player.vx = player.vx + player.gax * 300
		player.vy = player.vy + player.gay * 450
		
		-- Reset some varaibles to their pre-grapple state. Reset to Stage 1.
		player.grappleStage = 0
		player.canMove = true
		TEsound.play(sound.player.launch)
	end
	
end

function player.mouse(x,y,button)
	if button == 'l' then player.shoot(x,y) end
	if button == 'r' then player.grapple(x,y) end
end

function player.keyboard(key)
	if key == 'f' then player.flashing = not player.flashing player.health = player.health - 1 end
end